# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .apps import (
    AppsResource,
    AsyncAppsResource,
    AppsResourceWithRawResponse,
    AsyncAppsResourceWithRawResponse,
    AppsResourceWithStreamingResponse,
    AsyncAppsResourceWithStreamingResponse,
)
from .plans import (
    PlansResource,
    AsyncPlansResource,
    PlansResourceWithRawResponse,
    AsyncPlansResourceWithRawResponse,
    PlansResourceWithStreamingResponse,
    AsyncPlansResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .forums import (
    ForumsResource,
    AsyncForumsResource,
    ForumsResourceWithRawResponse,
    AsyncForumsResourceWithRawResponse,
    ForumsResourceWithStreamingResponse,
    AsyncForumsResourceWithStreamingResponse,
)
from .entries import (
    EntriesResource,
    AsyncEntriesResource,
    EntriesResourceWithRawResponse,
    AsyncEntriesResourceWithRawResponse,
    EntriesResourceWithStreamingResponse,
    AsyncEntriesResourceWithStreamingResponse,
)
from .members import (
    MembersResource,
    AsyncMembersResource,
    MembersResourceWithRawResponse,
    AsyncMembersResourceWithRawResponse,
    MembersResourceWithStreamingResponse,
    AsyncMembersResourceWithStreamingResponse,
)
from .invoices import (
    InvoicesResource,
    AsyncInvoicesResource,
    InvoicesResourceWithRawResponse,
    AsyncInvoicesResourceWithRawResponse,
    InvoicesResourceWithStreamingResponse,
    AsyncInvoicesResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)
from .payments import (
    PaymentsResource,
    AsyncPaymentsResource,
    PaymentsResourceWithRawResponse,
    AsyncPaymentsResourceWithRawResponse,
    PaymentsResourceWithStreamingResponse,
    AsyncPaymentsResourceWithStreamingResponse,
)
from .products import (
    ProductsResource,
    AsyncProductsResource,
    ProductsResourceWithRawResponse,
    AsyncProductsResourceWithRawResponse,
    ProductsResourceWithStreamingResponse,
    AsyncProductsResourceWithStreamingResponse,
)
from .webhooks import WebhooksResource, AsyncWebhooksResource
from .companies import (
    CompaniesResource,
    AsyncCompaniesResource,
    CompaniesResourceWithRawResponse,
    AsyncCompaniesResourceWithRawResponse,
    CompaniesResourceWithStreamingResponse,
    AsyncCompaniesResourceWithStreamingResponse,
)
from .reactions import (
    ReactionsResource,
    AsyncReactionsResource,
    ReactionsResourceWithRawResponse,
    AsyncReactionsResourceWithRawResponse,
    ReactionsResourceWithStreamingResponse,
    AsyncReactionsResourceWithStreamingResponse,
)
from .shipments import (
    ShipmentsResource,
    AsyncShipmentsResource,
    ShipmentsResourceWithRawResponse,
    AsyncShipmentsResourceWithRawResponse,
    ShipmentsResourceWithStreamingResponse,
    AsyncShipmentsResourceWithStreamingResponse,
)
from .transfers import (
    TransfersResource,
    AsyncTransfersResource,
    TransfersResourceWithRawResponse,
    AsyncTransfersResourceWithRawResponse,
    TransfersResourceWithStreamingResponse,
    AsyncTransfersResourceWithStreamingResponse,
)
from .app_builds import (
    AppBuildsResource,
    AsyncAppBuildsResource,
    AppBuildsResourceWithRawResponse,
    AsyncAppBuildsResourceWithRawResponse,
    AppBuildsResourceWithStreamingResponse,
    AsyncAppBuildsResourceWithStreamingResponse,
)
from .experiences import (
    ExperiencesResource,
    AsyncExperiencesResource,
    ExperiencesResourceWithRawResponse,
    AsyncExperiencesResourceWithRawResponse,
    ExperiencesResourceWithStreamingResponse,
    AsyncExperiencesResourceWithStreamingResponse,
)
from .forum_posts import (
    ForumPostsResource,
    AsyncForumPostsResource,
    ForumPostsResourceWithRawResponse,
    AsyncForumPostsResourceWithRawResponse,
    ForumPostsResourceWithStreamingResponse,
    AsyncForumPostsResourceWithStreamingResponse,
)
from .memberships import (
    MembershipsResource,
    AsyncMembershipsResource,
    MembershipsResourceWithRawResponse,
    AsyncMembershipsResourceWithRawResponse,
    MembershipsResourceWithStreamingResponse,
    AsyncMembershipsResourceWithStreamingResponse,
)
from .chat_channels import (
    ChatChannelsResource,
    AsyncChatChannelsResource,
    ChatChannelsResourceWithRawResponse,
    AsyncChatChannelsResourceWithRawResponse,
    ChatChannelsResourceWithStreamingResponse,
    AsyncChatChannelsResourceWithStreamingResponse,
)
from .ledger_accounts import (
    LedgerAccountsResource,
    AsyncLedgerAccountsResource,
    LedgerAccountsResourceWithRawResponse,
    AsyncLedgerAccountsResourceWithRawResponse,
    LedgerAccountsResourceWithStreamingResponse,
    AsyncLedgerAccountsResourceWithStreamingResponse,
)
from .authorized_users import (
    AuthorizedUsersResource,
    AsyncAuthorizedUsersResource,
    AuthorizedUsersResourceWithRawResponse,
    AsyncAuthorizedUsersResourceWithRawResponse,
    AuthorizedUsersResourceWithStreamingResponse,
    AsyncAuthorizedUsersResourceWithStreamingResponse,
)
from .support_channels import (
    SupportChannelsResource,
    AsyncSupportChannelsResource,
    SupportChannelsResourceWithRawResponse,
    AsyncSupportChannelsResourceWithRawResponse,
    SupportChannelsResourceWithStreamingResponse,
    AsyncSupportChannelsResourceWithStreamingResponse,
)
from .checkout_configurations import (
    CheckoutConfigurationsResource,
    AsyncCheckoutConfigurationsResource,
    CheckoutConfigurationsResourceWithRawResponse,
    AsyncCheckoutConfigurationsResourceWithRawResponse,
    CheckoutConfigurationsResourceWithStreamingResponse,
    AsyncCheckoutConfigurationsResourceWithStreamingResponse,
)
from .course_lesson_interactions import (
    CourseLessonInteractionsResource,
    AsyncCourseLessonInteractionsResource,
    CourseLessonInteractionsResourceWithRawResponse,
    AsyncCourseLessonInteractionsResourceWithRawResponse,
    CourseLessonInteractionsResourceWithStreamingResponse,
    AsyncCourseLessonInteractionsResourceWithStreamingResponse,
)

__all__ = [
    "AppsResource",
    "AsyncAppsResource",
    "AppsResourceWithRawResponse",
    "AsyncAppsResourceWithRawResponse",
    "AppsResourceWithStreamingResponse",
    "AsyncAppsResourceWithStreamingResponse",
    "InvoicesResource",
    "AsyncInvoicesResource",
    "InvoicesResourceWithRawResponse",
    "AsyncInvoicesResourceWithRawResponse",
    "InvoicesResourceWithStreamingResponse",
    "AsyncInvoicesResourceWithStreamingResponse",
    "CourseLessonInteractionsResource",
    "AsyncCourseLessonInteractionsResource",
    "CourseLessonInteractionsResourceWithRawResponse",
    "AsyncCourseLessonInteractionsResourceWithRawResponse",
    "CourseLessonInteractionsResourceWithStreamingResponse",
    "AsyncCourseLessonInteractionsResourceWithStreamingResponse",
    "ProductsResource",
    "AsyncProductsResource",
    "ProductsResourceWithRawResponse",
    "AsyncProductsResourceWithRawResponse",
    "ProductsResourceWithStreamingResponse",
    "AsyncProductsResourceWithStreamingResponse",
    "CompaniesResource",
    "AsyncCompaniesResource",
    "CompaniesResourceWithRawResponse",
    "AsyncCompaniesResourceWithRawResponse",
    "CompaniesResourceWithStreamingResponse",
    "AsyncCompaniesResourceWithStreamingResponse",
    "WebhooksResource",
    "AsyncWebhooksResource",
    "PlansResource",
    "AsyncPlansResource",
    "PlansResourceWithRawResponse",
    "AsyncPlansResourceWithRawResponse",
    "PlansResourceWithStreamingResponse",
    "AsyncPlansResourceWithStreamingResponse",
    "EntriesResource",
    "AsyncEntriesResource",
    "EntriesResourceWithRawResponse",
    "AsyncEntriesResourceWithRawResponse",
    "EntriesResourceWithStreamingResponse",
    "AsyncEntriesResourceWithStreamingResponse",
    "ForumPostsResource",
    "AsyncForumPostsResource",
    "ForumPostsResourceWithRawResponse",
    "AsyncForumPostsResourceWithRawResponse",
    "ForumPostsResourceWithStreamingResponse",
    "AsyncForumPostsResourceWithStreamingResponse",
    "TransfersResource",
    "AsyncTransfersResource",
    "TransfersResourceWithRawResponse",
    "AsyncTransfersResourceWithRawResponse",
    "TransfersResourceWithStreamingResponse",
    "AsyncTransfersResourceWithStreamingResponse",
    "LedgerAccountsResource",
    "AsyncLedgerAccountsResource",
    "LedgerAccountsResourceWithRawResponse",
    "AsyncLedgerAccountsResourceWithRawResponse",
    "LedgerAccountsResourceWithStreamingResponse",
    "AsyncLedgerAccountsResourceWithStreamingResponse",
    "MembershipsResource",
    "AsyncMembershipsResource",
    "MembershipsResourceWithRawResponse",
    "AsyncMembershipsResourceWithRawResponse",
    "MembershipsResourceWithStreamingResponse",
    "AsyncMembershipsResourceWithStreamingResponse",
    "AuthorizedUsersResource",
    "AsyncAuthorizedUsersResource",
    "AuthorizedUsersResourceWithRawResponse",
    "AsyncAuthorizedUsersResourceWithRawResponse",
    "AuthorizedUsersResourceWithStreamingResponse",
    "AsyncAuthorizedUsersResourceWithStreamingResponse",
    "AppBuildsResource",
    "AsyncAppBuildsResource",
    "AppBuildsResourceWithRawResponse",
    "AsyncAppBuildsResourceWithRawResponse",
    "AppBuildsResourceWithStreamingResponse",
    "AsyncAppBuildsResourceWithStreamingResponse",
    "ShipmentsResource",
    "AsyncShipmentsResource",
    "ShipmentsResourceWithRawResponse",
    "AsyncShipmentsResourceWithRawResponse",
    "ShipmentsResourceWithStreamingResponse",
    "AsyncShipmentsResourceWithStreamingResponse",
    "CheckoutConfigurationsResource",
    "AsyncCheckoutConfigurationsResource",
    "CheckoutConfigurationsResourceWithRawResponse",
    "AsyncCheckoutConfigurationsResourceWithRawResponse",
    "CheckoutConfigurationsResourceWithStreamingResponse",
    "AsyncCheckoutConfigurationsResourceWithStreamingResponse",
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
    "ChatChannelsResource",
    "AsyncChatChannelsResource",
    "ChatChannelsResourceWithRawResponse",
    "AsyncChatChannelsResourceWithRawResponse",
    "ChatChannelsResourceWithStreamingResponse",
    "AsyncChatChannelsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "PaymentsResource",
    "AsyncPaymentsResource",
    "PaymentsResourceWithRawResponse",
    "AsyncPaymentsResourceWithRawResponse",
    "PaymentsResourceWithStreamingResponse",
    "AsyncPaymentsResourceWithStreamingResponse",
    "SupportChannelsResource",
    "AsyncSupportChannelsResource",
    "SupportChannelsResourceWithRawResponse",
    "AsyncSupportChannelsResourceWithRawResponse",
    "SupportChannelsResourceWithStreamingResponse",
    "AsyncSupportChannelsResourceWithStreamingResponse",
    "ExperiencesResource",
    "AsyncExperiencesResource",
    "ExperiencesResourceWithRawResponse",
    "AsyncExperiencesResourceWithRawResponse",
    "ExperiencesResourceWithStreamingResponse",
    "AsyncExperiencesResourceWithStreamingResponse",
    "ReactionsResource",
    "AsyncReactionsResource",
    "ReactionsResourceWithRawResponse",
    "AsyncReactionsResourceWithRawResponse",
    "ReactionsResourceWithStreamingResponse",
    "AsyncReactionsResourceWithStreamingResponse",
    "MembersResource",
    "AsyncMembersResource",
    "MembersResourceWithRawResponse",
    "AsyncMembersResourceWithRawResponse",
    "MembersResourceWithStreamingResponse",
    "AsyncMembersResourceWithStreamingResponse",
    "ForumsResource",
    "AsyncForumsResource",
    "ForumsResourceWithRawResponse",
    "AsyncForumsResourceWithRawResponse",
    "ForumsResourceWithStreamingResponse",
    "AsyncForumsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["AppCreateParams"]


class AppCreateParams(TypedDict, total=False):
    company_id: Required[str]
    """The ID of the company to create the app for"""

    name: Required[str]
    """The name of the app to be created"""

    base_url: Optional[str]
    """The base URL of the app to be created"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TaxType"]

TaxType: TypeAlias = Literal["inclusive", "exclusive", "unspecified"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import invoice_list_params, invoice_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.invoice import Invoice
from ..types.shared.direction import Direction
from ..types.invoice_void_response import InvoiceVoidResponse
from ..types.invoice_create_response import InvoiceCreateResponse
from ..types.shared.collection_method import CollectionMethod
from ..types.shared.invoice_list_item import InvoiceListItem

__all__ = ["InvoicesResource", "AsyncInvoicesResource"]


class InvoicesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return InvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return InvoicesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        collection_method: CollectionMethod,
        company_id: str,
        due_date: Union[str, datetime],
        plan: invoice_create_params.Plan,
        charge_buyer_fee: Optional[bool] | Omit = omit,
        customer_name: Optional[str] | Omit = omit,
        email_address: Optional[str] | Omit = omit,
        member_id: Optional[str] | Omit = omit,
        payment_token_id: Optional[str] | Omit = omit,
        product: Optional[invoice_create_params.Product] | Omit = omit,
        product_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceCreateResponse:
        """
        Creates an invoice

        Required permissions:

        - `invoice:create`
        - `plan:basic:read`

        Args:
          collection_method: The method of collection for this invoice. If using charge_automatically, you
              must provide a payment_token.

          company_id: The company ID to create this invoice for.

          due_date: The date the invoice is due, if applicable.

          plan: The properties of the plan to create for this invoice.

          charge_buyer_fee: Whether or not to charge the customer a buyer fee.

          customer_name: The name of the customer to create this invoice for. This is required if you
              want to create an invoice for a customer who does not have a member of your
              company yet.

          email_address: The email address to create this invoice for. This is required if you want to
              create an invoice for a user who does not have a member of your company yet.

          member_id: The member ID to create this invoice for. Include this if you want to create an
              invoice for an existing member. If you do not have a member ID, you must provide
              an email_address and customer_name.

          payment_token_id: The payment token ID to use for this invoice. If using charge_automatically, you
              must provide a payment_token.

          product: The properties of the product to create for this invoice. Include this if you
              want to create an invoice for a new product.

          product_id: The product ID to create this invoice for. Include this if you want to create an
              invoice for an existing product.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/invoices",
            body=maybe_transform(
                {
                    "collection_method": collection_method,
                    "company_id": company_id,
                    "due_date": due_date,
                    "plan": plan,
                    "charge_buyer_fee": charge_buyer_fee,
                    "customer_name": customer_name,
                    "email_address": email_address,
                    "member_id": member_id,
                    "payment_token_id": payment_token_id,
                    "product": product,
                    "product_id": product_id,
                },
                invoice_create_params.InvoiceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Invoice:
        """
        Retrieves an invoice by ID or token

        Required permissions:

        - `invoice:basic:read`
        - `plan:basic:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/invoices/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Invoice,
        )

    def list(
        self,
        *,
        company_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        direction: Optional[Direction] | Omit = omit,
        filters: Optional[invoice_list_params.Filters] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        order: Optional[Literal["id", "created_at", "due_date"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[InvoiceListItem]:
        """
        Lists invoices

        Required permissions:

        - `invoice:basic:read`
        - `plan:basic:read`

        Args:
          company_id: The ID of the company to list invoices for

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          direction: The direction of the sort.

          filters: The filters to apply to the invoices

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          order: Which columns can be used to sort.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/invoices",
            page=SyncCursorPage[InvoiceListItem],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "after": after,
                        "before": before,
                        "direction": direction,
                        "filters": filters,
                        "first": first,
                        "last": last,
                        "order": order,
                    },
                    invoice_list_params.InvoiceListParams,
                ),
            ),
            model=InvoiceListItem,
        )

    def void(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceVoidResponse:
        """
        Void an invoice

        Required permissions:

        - `invoice:update`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/invoices/{id}/void",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceVoidResponse,
        )


class AsyncInvoicesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInvoicesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInvoicesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInvoicesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncInvoicesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        collection_method: CollectionMethod,
        company_id: str,
        due_date: Union[str, datetime],
        plan: invoice_create_params.Plan,
        charge_buyer_fee: Optional[bool] | Omit = omit,
        customer_name: Optional[str] | Omit = omit,
        email_address: Optional[str] | Omit = omit,
        member_id: Optional[str] | Omit = omit,
        payment_token_id: Optional[str] | Omit = omit,
        product: Optional[invoice_create_params.Product] | Omit = omit,
        product_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceCreateResponse:
        """
        Creates an invoice

        Required permissions:

        - `invoice:create`
        - `plan:basic:read`

        Args:
          collection_method: The method of collection for this invoice. If using charge_automatically, you
              must provide a payment_token.

          company_id: The company ID to create this invoice for.

          due_date: The date the invoice is due, if applicable.

          plan: The properties of the plan to create for this invoice.

          charge_buyer_fee: Whether or not to charge the customer a buyer fee.

          customer_name: The name of the customer to create this invoice for. This is required if you
              want to create an invoice for a customer who does not have a member of your
              company yet.

          email_address: The email address to create this invoice for. This is required if you want to
              create an invoice for a user who does not have a member of your company yet.

          member_id: The member ID to create this invoice for. Include this if you want to create an
              invoice for an existing member. If you do not have a member ID, you must provide
              an email_address and customer_name.

          payment_token_id: The payment token ID to use for this invoice. If using charge_automatically, you
              must provide a payment_token.

          product: The properties of the product to create for this invoice. Include this if you
              want to create an invoice for a new product.

          product_id: The product ID to create this invoice for. Include this if you want to create an
              invoice for an existing product.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/invoices",
            body=await async_maybe_transform(
                {
                    "collection_method": collection_method,
                    "company_id": company_id,
                    "due_date": due_date,
                    "plan": plan,
                    "charge_buyer_fee": charge_buyer_fee,
                    "customer_name": customer_name,
                    "email_address": email_address,
                    "member_id": member_id,
                    "payment_token_id": payment_token_id,
                    "product": product,
                    "product_id": product_id,
                },
                invoice_create_params.InvoiceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Invoice:
        """
        Retrieves an invoice by ID or token

        Required permissions:

        - `invoice:basic:read`
        - `plan:basic:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/invoices/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Invoice,
        )

    def list(
        self,
        *,
        company_id: str,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        direction: Optional[Direction] | Omit = omit,
        filters: Optional[invoice_list_params.Filters] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        order: Optional[Literal["id", "created_at", "due_date"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[InvoiceListItem, AsyncCursorPage[InvoiceListItem]]:
        """
        Lists invoices

        Required permissions:

        - `invoice:basic:read`
        - `plan:basic:read`

        Args:
          company_id: The ID of the company to list invoices for

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          direction: The direction of the sort.

          filters: The filters to apply to the invoices

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          order: Which columns can be used to sort.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/invoices",
            page=AsyncCursorPage[InvoiceListItem],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "after": after,
                        "before": before,
                        "direction": direction,
                        "filters": filters,
                        "first": first,
                        "last": last,
                        "order": order,
                    },
                    invoice_list_params.InvoiceListParams,
                ),
            ),
            model=InvoiceListItem,
        )

    async def void(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InvoiceVoidResponse:
        """
        Void an invoice

        Required permissions:

        - `invoice:update`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/invoices/{id}/void",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InvoiceVoidResponse,
        )


class InvoicesResourceWithRawResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

        self.create = to_raw_response_wrapper(
            invoices.create,
        )
        self.retrieve = to_raw_response_wrapper(
            invoices.retrieve,
        )
        self.list = to_raw_response_wrapper(
            invoices.list,
        )
        self.void = to_raw_response_wrapper(
            invoices.void,
        )


class AsyncInvoicesResourceWithRawResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

        self.create = async_to_raw_response_wrapper(
            invoices.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            invoices.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            invoices.list,
        )
        self.void = async_to_raw_response_wrapper(
            invoices.void,
        )


class InvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: InvoicesResource) -> None:
        self._invoices = invoices

        self.create = to_streamed_response_wrapper(
            invoices.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            invoices.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            invoices.list,
        )
        self.void = to_streamed_response_wrapper(
            invoices.void,
        )


class AsyncInvoicesResourceWithStreamingResponse:
    def __init__(self, invoices: AsyncInvoicesResource) -> None:
        self._invoices = invoices

        self.create = async_to_streamed_response_wrapper(
            invoices.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            invoices.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            invoices.list,
        )
        self.void = async_to_streamed_response_wrapper(
            invoices.void,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ExperienceAttachParams"]


class ExperienceAttachParams(TypedDict, total=False):
    product_id: Required[str]
    """The ID of the Access Pass to add the Experience to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .shared.entry_status import EntryStatus

__all__ = ["EntryListResponse", "Plan", "Product", "User"]


class Plan(BaseModel):
    id: str
    """The internal ID of the plan."""


class Product(BaseModel):
    id: str
    """The internal ID of the public product."""

    title: str
    """The title of the product. Use for Whop 4.0."""


class User(BaseModel):
    id: str
    """The internal ID of the user."""

    email: Optional[str] = None
    """The email of the user"""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class EntryListResponse(BaseModel):
    id: str
    """The internal ID of the entry."""

    created_at: Optional[datetime] = None
    """When the entry was created."""

    plan: Optional[Plan] = None
    """The waitlist plan the entry if for."""

    product: Optional[Product] = None
    """The access pass tied to this entry, if there is one."""

    status: EntryStatus
    """The status of the entry."""

    user: User
    """The user who created the entry."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .shared.currency import Currency
from .shared.direction import Direction
from .shared.receipt_status import ReceiptStatus
from .shared.friendly_receipt_status import FriendlyReceiptStatus

__all__ = ["PaymentListParams"]


class PaymentListParams(TypedDict, total=False):
    company_id: Required[str]
    """The ID of the company to list payments for"""

    after: Optional[str]
    """Returns the elements in the list that come after the specified cursor."""

    before: Optional[str]
    """Returns the elements in the list that come before the specified cursor."""

    billing_reasons: Optional[
        List[
            Literal[
                "subscription_create", "subscription_cycle", "subscription_update", "one_time", "manual", "subscription"
            ]
        ]
    ]
    """The billing reason for the payment"""

    created_after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """The minimum creation date to filter by"""

    created_before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """The maximum creation date to filter by"""

    currencies: Optional[List[Currency]]
    """The currency of the payment."""

    direction: Optional[Direction]
    """The direction of the sort."""

    first: Optional[int]
    """Returns the first _n_ elements from the list."""

    include_free: Optional[bool]
    """Whether to include free payments."""

    last: Optional[int]
    """Returns the last _n_ elements from the list."""

    order: Optional[Literal["final_amount", "created_at", "paid_at"]]
    """The order to sort the results by."""

    plan_ids: Optional[SequenceNotStr[str]]
    """A specific plan."""

    product_ids: Optional[SequenceNotStr[str]]
    """A specific product."""

    statuses: Optional[List[ReceiptStatus]]
    """The status of the payment."""

    substatuses: Optional[List[FriendlyReceiptStatus]]
    """The substatus of the payment."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import (
    membership_list_params,
    membership_pause_params,
    membership_cancel_params,
    membership_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.direction import Direction
from ..types.shared.membership import Membership
from ..types.membership_list_response import MembershipListResponse
from ..types.shared.membership_status import MembershipStatus

__all__ = ["MembershipsResource", "AsyncMembershipsResource"]


class MembershipsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MembershipsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return MembershipsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MembershipsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return MembershipsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Retrieves a membership by ID or license key

        Required permissions:

        - `member:basic:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/memberships/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )

    def update(
        self,
        id: str,
        *,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Update a membership

        Required permissions:

        - `member:manage`
        - `member:basic:read`

        Args:
          metadata: The metadata to update the membership with.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/memberships/{id}",
            body=maybe_transform({"metadata": metadata}, membership_update_params.MembershipUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )

    def list(
        self,
        *,
        company_id: str,
        access_pass_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        cancel_options: Optional[
            List[
                Literal[
                    "too_expensive",
                    "switching",
                    "missing_features",
                    "technical_issues",
                    "bad_experience",
                    "other",
                    "testing",
                ]
            ]
        ]
        | Omit = omit,
        created_after: Union[str, datetime, None] | Omit = omit,
        created_before: Union[str, datetime, None] | Omit = omit,
        direction: Optional[Direction] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        order: Optional[Literal["id", "created_at", "status", "canceled_at", "date_joined", "total_spend"]]
        | Omit = omit,
        plan_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        promo_code_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        statuses: Optional[List[MembershipStatus]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[MembershipListResponse]:
        """
        Lists memberships

        Required permissions:

        - `member:basic:read`

        Args:
          company_id: The ID of the company to list memberships for

          access_pass_ids: The access pass IDs to filter the memberships by

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          cancel_options: The cancel options to filter the memberships by

          created_after: The minimum creation date to filter by

          created_before: The maximum creation date to filter by

          direction: The direction of the sort.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          order: Which columns can be used to sort.

          plan_ids: The plan IDs to filter the memberships by

          promo_code_ids: The promo code IDs to filter the memberships by

          statuses: The membership status to filter the memberships by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/memberships",
            page=SyncCursorPage[MembershipListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "access_pass_ids": access_pass_ids,
                        "after": after,
                        "before": before,
                        "cancel_options": cancel_options,
                        "created_after": created_after,
                        "created_before": created_before,
                        "direction": direction,
                        "first": first,
                        "last": last,
                        "order": order,
                        "plan_ids": plan_ids,
                        "promo_code_ids": promo_code_ids,
                        "statuses": statuses,
                    },
                    membership_list_params.MembershipListParams,
                ),
            ),
            model=MembershipListResponse,
        )

    def cancel(
        self,
        id: str,
        *,
        cancellation_mode: Optional[Literal["at_period_end", "immediate"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Cancels a membership either immediately or at the end of the current billing
        period

        Required permissions:

        - `member:manage`
        - `member:basic:read`

        Args:
          cancellation_mode: The mode of cancellation for a membership

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/memberships/{id}/cancel",
            body=maybe_transform(
                {"cancellation_mode": cancellation_mode}, membership_cancel_params.MembershipCancelParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )

    def pause(
        self,
        id: str,
        *,
        void_payments: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Pauses a membership's payments

        Required permissions:

        - `member:manage`
        - `member:basic:read`

        Args:
          void_payments: Whether to void past_due payments associated with the membership to prevent
              future payment attempts.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/memberships/{id}/pause",
            body=maybe_transform({"void_payments": void_payments}, membership_pause_params.MembershipPauseParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )

    def resume(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Resumes a membership's payments

        Required permissions:

        - `member:manage`
        - `member:basic:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/memberships/{id}/resume",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )


class AsyncMembershipsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMembershipsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMembershipsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMembershipsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncMembershipsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Retrieves a membership by ID or license key

        Required permissions:

        - `member:basic:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/memberships/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )

    async def update(
        self,
        id: str,
        *,
        metadata: Optional[Dict[str, object]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Update a membership

        Required permissions:

        - `member:manage`
        - `member:basic:read`

        Args:
          metadata: The metadata to update the membership with.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/memberships/{id}",
            body=await async_maybe_transform({"metadata": metadata}, membership_update_params.MembershipUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )

    def list(
        self,
        *,
        company_id: str,
        access_pass_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        cancel_options: Optional[
            List[
                Literal[
                    "too_expensive",
                    "switching",
                    "missing_features",
                    "technical_issues",
                    "bad_experience",
                    "other",
                    "testing",
                ]
            ]
        ]
        | Omit = omit,
        created_after: Union[str, datetime, None] | Omit = omit,
        created_before: Union[str, datetime, None] | Omit = omit,
        direction: Optional[Direction] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        order: Optional[Literal["id", "created_at", "status", "canceled_at", "date_joined", "total_spend"]]
        | Omit = omit,
        plan_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        promo_code_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        statuses: Optional[List[MembershipStatus]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MembershipListResponse, AsyncCursorPage[MembershipListResponse]]:
        """
        Lists memberships

        Required permissions:

        - `member:basic:read`

        Args:
          company_id: The ID of the company to list memberships for

          access_pass_ids: The access pass IDs to filter the memberships by

          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          cancel_options: The cancel options to filter the memberships by

          created_after: The minimum creation date to filter by

          created_before: The maximum creation date to filter by

          direction: The direction of the sort.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          order: Which columns can be used to sort.

          plan_ids: The plan IDs to filter the memberships by

          promo_code_ids: The promo code IDs to filter the memberships by

          statuses: The membership status to filter the memberships by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/memberships",
            page=AsyncCursorPage[MembershipListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "company_id": company_id,
                        "access_pass_ids": access_pass_ids,
                        "after": after,
                        "before": before,
                        "cancel_options": cancel_options,
                        "created_after": created_after,
                        "created_before": created_before,
                        "direction": direction,
                        "first": first,
                        "last": last,
                        "order": order,
                        "plan_ids": plan_ids,
                        "promo_code_ids": promo_code_ids,
                        "statuses": statuses,
                    },
                    membership_list_params.MembershipListParams,
                ),
            ),
            model=MembershipListResponse,
        )

    async def cancel(
        self,
        id: str,
        *,
        cancellation_mode: Optional[Literal["at_period_end", "immediate"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Cancels a membership either immediately or at the end of the current billing
        period

        Required permissions:

        - `member:manage`
        - `member:basic:read`

        Args:
          cancellation_mode: The mode of cancellation for a membership

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/memberships/{id}/cancel",
            body=await async_maybe_transform(
                {"cancellation_mode": cancellation_mode}, membership_cancel_params.MembershipCancelParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )

    async def pause(
        self,
        id: str,
        *,
        void_payments: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Pauses a membership's payments

        Required permissions:

        - `member:manage`
        - `member:basic:read`

        Args:
          void_payments: Whether to void past_due payments associated with the membership to prevent
              future payment attempts.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/memberships/{id}/pause",
            body=await async_maybe_transform(
                {"void_payments": void_payments}, membership_pause_params.MembershipPauseParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )

    async def resume(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Membership:
        """
        Resumes a membership's payments

        Required permissions:

        - `member:manage`
        - `member:basic:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/memberships/{id}/resume",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Membership,
        )


class MembershipsResourceWithRawResponse:
    def __init__(self, memberships: MembershipsResource) -> None:
        self._memberships = memberships

        self.retrieve = to_raw_response_wrapper(
            memberships.retrieve,
        )
        self.update = to_raw_response_wrapper(
            memberships.update,
        )
        self.list = to_raw_response_wrapper(
            memberships.list,
        )
        self.cancel = to_raw_response_wrapper(
            memberships.cancel,
        )
        self.pause = to_raw_response_wrapper(
            memberships.pause,
        )
        self.resume = to_raw_response_wrapper(
            memberships.resume,
        )


class AsyncMembershipsResourceWithRawResponse:
    def __init__(self, memberships: AsyncMembershipsResource) -> None:
        self._memberships = memberships

        self.retrieve = async_to_raw_response_wrapper(
            memberships.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            memberships.update,
        )
        self.list = async_to_raw_response_wrapper(
            memberships.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            memberships.cancel,
        )
        self.pause = async_to_raw_response_wrapper(
            memberships.pause,
        )
        self.resume = async_to_raw_response_wrapper(
            memberships.resume,
        )


class MembershipsResourceWithStreamingResponse:
    def __init__(self, memberships: MembershipsResource) -> None:
        self._memberships = memberships

        self.retrieve = to_streamed_response_wrapper(
            memberships.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            memberships.update,
        )
        self.list = to_streamed_response_wrapper(
            memberships.list,
        )
        self.cancel = to_streamed_response_wrapper(
            memberships.cancel,
        )
        self.pause = to_streamed_response_wrapper(
            memberships.pause,
        )
        self.resume = to_streamed_response_wrapper(
            memberships.resume,
        )


class AsyncMembershipsResourceWithStreamingResponse:
    def __init__(self, memberships: AsyncMembershipsResource) -> None:
        self._memberships = memberships

        self.retrieve = async_to_streamed_response_wrapper(
            memberships.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            memberships.update,
        )
        self.list = async_to_streamed_response_wrapper(
            memberships.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            memberships.cancel,
        )
        self.pause = async_to_streamed_response_wrapper(
            memberships.pause,
        )
        self.resume = async_to_streamed_response_wrapper(
            memberships.resume,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["CourseLessonInteraction", "Course", "CourseExperience", "Lesson", "User"]


class CourseExperience(BaseModel):
    id: str
    """The unique ID representing this experience"""


class Course(BaseModel):
    id: str
    """The ID of the course. Looks like cors_XXX"""

    experience: CourseExperience
    """The experience that the course belongs to"""

    title: Optional[str] = None
    """The title of the course"""


class Lesson(BaseModel):
    id: str
    """The ID of the lesson"""

    title: str
    """The title of the lesson"""


class User(BaseModel):
    id: str
    """The internal ID of the user."""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class CourseLessonInteraction(BaseModel):
    id: str
    """The ID of the lesson interaction"""

    completed: bool
    """Whether the lesson has been completed by the user"""

    course: Course
    """The course for this lesson interaction"""

    created_at: datetime
    """When the interaction was created"""

    lesson: Lesson
    """The lesson this interaction is for"""

    user: User
    """The user who interacted with the lesson"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["GlobalAffiliateStatus"]

GlobalAffiliateStatus: TypeAlias = Literal["enabled", "disabled"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["PromoType"]

PromoType: TypeAlias = Literal["percentage", "flat_amount"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from whop_sdk import Whop, AsyncWhop
from tests.utils import assert_matches_type
from whop_sdk.types import UserRetrieveResponse, UserCheckAccessResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Whop) -> None:
        user = client.users.retrieve(
            "user_xxxxxxxxxxxxx",
        )
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Whop) -> None:
        response = client.users.with_raw_response.retrieve(
            "user_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Whop) -> None:
        with client.users.with_streaming_response.retrieve(
            "user_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserRetrieveResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.users.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_check_access(self, client: Whop) -> None:
        user = client.users.check_access(
            resource_id="resource_id",
            id="user_xxxxxxxxxxxxx",
        )
        assert_matches_type(UserCheckAccessResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_check_access(self, client: Whop) -> None:
        response = client.users.with_raw_response.check_access(
            resource_id="resource_id",
            id="user_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserCheckAccessResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_check_access(self, client: Whop) -> None:
        with client.users.with_streaming_response.check_access(
            resource_id="resource_id",
            id="user_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserCheckAccessResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_check_access(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.users.with_raw_response.check_access(
                resource_id="resource_id",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource_id` but received ''"):
            client.users.with_raw_response.check_access(
                resource_id="",
                id="user_xxxxxxxxxxxxx",
            )


class TestAsyncUsers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncWhop) -> None:
        user = await async_client.users.retrieve(
            "user_xxxxxxxxxxxxx",
        )
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncWhop) -> None:
        response = await async_client.users.with_raw_response.retrieve(
            "user_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncWhop) -> None:
        async with async_client.users.with_streaming_response.retrieve(
            "user_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserRetrieveResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.users.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_check_access(self, async_client: AsyncWhop) -> None:
        user = await async_client.users.check_access(
            resource_id="resource_id",
            id="user_xxxxxxxxxxxxx",
        )
        assert_matches_type(UserCheckAccessResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_check_access(self, async_client: AsyncWhop) -> None:
        response = await async_client.users.with_raw_response.check_access(
            resource_id="resource_id",
            id="user_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserCheckAccessResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_check_access(self, async_client: AsyncWhop) -> None:
        async with async_client.users.with_streaming_response.check_access(
            resource_id="resource_id",
            id="user_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserCheckAccessResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_check_access(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.users.with_raw_response.check_access(
                resource_id="resource_id",
                id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `resource_id` but received ''"):
            await async_client.users.with_raw_response.check_access(
                resource_id="",
                id="user_xxxxxxxxxxxxx",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["InvoiceStatus"]

InvoiceStatus: TypeAlias = Literal["open", "paid", "past_due", "void"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["EmailNotificationPreferences"]

EmailNotificationPreferences: TypeAlias = Literal["all_admin_posts", "only_weekly_summary", "none"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .shared.authorized_user_roles import AuthorizedUserRoles

__all__ = ["AuthorizedUserListResponse", "User"]


class User(BaseModel):
    id: str
    """The internal ID of the user."""

    email: Optional[str] = None
    """The email of the user"""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class AuthorizedUserListResponse(BaseModel):
    id: str
    """A unique ID representing the authorized user object."""

    role: AuthorizedUserRoles
    """The role of the authorized user in the company."""

    user: User
    """The user associated with the authorized user."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .app import App as App
from .plan import Plan as Plan
from .entry import Entry as Entry
from .forum import Forum as Forum
from .company import Company as Company
from .invoice import Invoice as Invoice
from .message import Message as Message
from .payment import Payment as Payment
from .product import Product as Product
from .currency import Currency as Currency
from .reaction import Reaction as Reaction
from .shipment import Shipment as Shipment
from .tax_type import TaxType as TaxType
from .transfer import Transfer as Transfer
from .app_build import AppBuild as AppBuild
from .direction import Direction as Direction
from .page_info import PageInfo as PageInfo
from .plan_type import PlanType as PlanType
from .custom_cta import CustomCta as CustomCta
from .experience import Experience as Experience
from .forum_post import ForumPost as ForumPost
from .membership import Membership as Membership
from .promo_type import PromoType as PromoType
from .visibility import Visibility as Visibility
from .access_level import AccessLevel as AccessLevel
from .app_statuses import AppStatuses as AppStatuses
from .chat_channel import ChatChannel as ChatChannel
from .entry_status import EntryStatus as EntryStatus
from .who_can_post import WhoCanPost as WhoCanPost
from .app_view_type import AppViewType as AppViewType
from .who_can_react import WhoCanReact as WhoCanReact
from .business_types import BusinessTypes as BusinessTypes
from .dms_post_types import DmsPostTypes as DmsPostTypes
from .industry_types import IndustryTypes as IndustryTypes
from .invoice_status import InvoiceStatus as InvoiceStatus
from .receipt_status import ReceiptStatus as ReceiptStatus
from .release_method import ReleaseMethod as ReleaseMethod
from .member_statuses import MemberStatuses as MemberStatuses
from .shipment_status import ShipmentStatus as ShipmentStatus
from .support_channel import SupportChannel as SupportChannel
from .access_pass_type import AccessPassType as AccessPassType
from .shipment_carrier import ShipmentCarrier as ShipmentCarrier
from .collection_method import CollectionMethod as CollectionMethod
from .invoice_list_item import InvoiceListItem as InvoiceListItem
from .membership_status import MembershipStatus as MembershipStatus
from .product_list_item import ProductListItem as ProductListItem
from .visibility_filter import VisibilityFilter as VisibilityFilter
from .app_build_statuses import AppBuildStatuses as AppBuildStatuses
from .shipment_substatus import ShipmentSubstatus as ShipmentSubstatus
from .who_can_post_types import WhoCanPostTypes as WhoCanPostTypes
from .app_build_platforms import AppBuildPlatforms as AppBuildPlatforms
from .authorized_user_roles import AuthorizedUserRoles as AuthorizedUserRoles
from .who_can_comment_types import WhoCanCommentTypes as WhoCanCommentTypes
from .checkout_configuration import CheckoutConfiguration as CheckoutConfiguration
from .friendly_receipt_status import FriendlyReceiptStatus as FriendlyReceiptStatus
from .global_affiliate_status import GlobalAffiliateStatus as GlobalAffiliateStatus
from .course_lesson_interaction import CourseLessonInteraction as CourseLessonInteraction
from .member_most_recent_actions import MemberMostRecentActions as MemberMostRecentActions
from .email_notification_preferences import EmailNotificationPreferences as EmailNotificationPreferences
from .course_lesson_interaction_list_item import CourseLessonInteractionListItem as CourseLessonInteractionListItem



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import TypedDict

__all__ = ["MembershipUpdateParams"]


class MembershipUpdateParams(TypedDict, total=False):
    metadata: Optional[Dict[str, object]]
    """The metadata to update the membership with."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel
from .app_statuses import AppStatuses

__all__ = [
    "App",
    "APIKey",
    "Company",
    "Creator",
    "Icon",
    "RequestedPermission",
    "RequestedPermissionPermissionAction",
    "Stats",
]


class APIKey(BaseModel):
    id: str
    """The ID of this API key"""

    token: str
    """This is the API key used to authenticate requests"""

    created_at: datetime
    """When this API key was created at"""


class Company(BaseModel):
    id: str
    """The ID (tag) of the company."""

    title: str
    """The title of the company."""


class Creator(BaseModel):
    id: str
    """The internal ID of the user."""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    username: str
    """The username of the user from their Whop account."""


class Icon(BaseModel):
    url: Optional[str] = None
    """This is the URL you use to render optimized attachments on the client.

    This should be used for apps.
    """


class RequestedPermissionPermissionAction(BaseModel):
    action: str
    """The identifier of the action."""

    name: str
    """The human readable name of the action."""


class RequestedPermission(BaseModel):
    is_required: bool
    """Whether the action is required for the app to function."""

    justification: str
    """The reason for requesting the action."""

    permission_action: RequestedPermissionPermissionAction
    """The action that the app will request off of users when a user installs the app."""


class Stats(BaseModel):
    dau: int
    """
    This is the number of users that have spent time in this app in the last 24
    hours.
    """

    mau: int
    """
    This is the number of users that have spent time in this app in the last 28
    days.
    """

    time_spent_last24_hours: int
    """
    This how much time, in seconds, users have spent in this app in the last 24
    hours.
    """

    wau: int
    """
    This is the number of users that have spent time in this app in the last 7 days.
    """


class App(BaseModel):
    id: str
    """The ID of the app"""

    api_key: Optional[APIKey] = None
    """The API key for the app"""

    base_url: Optional[str] = None
    """The base url of the app"""

    company: Company
    """The company that owns the app"""

    creator: Creator
    """The creator of the app"""

    dashboard_path: Optional[str] = None
    """The path part for a specific view of the app.

    This is the template part of the url after the base domain. Eg:
    /experiences/[experienceId]
    """

    description: Optional[str] = None
    """The description of the app"""

    discover_path: Optional[str] = None
    """The path part for a specific view of the app.

    This is the template part of the url after the base domain. Eg:
    /experiences/[experienceId]
    """

    domain_id: str
    """The unique part of the proxied domain for this app.

    Used to generate the base url used to display the app inside the whop platform.
    Refers to the id part in the final url: https://{domain_id}.apps.whop.com
    """

    experience_path: Optional[str] = None
    """The path part for a specific view of the app.

    This is the template part of the url after the base domain. Eg:
    /experiences/[experienceId]
    """

    icon: Optional[Icon] = None
    """The icon for the app.

    This icon is shown on discovery, on the product page, on checkout, and as a
    default icon for the experiences.
    """

    name: str
    """The name of the app"""

    requested_permissions: List[RequestedPermission]
    """
    The set of permissions that an app requests to be granted when a user installs
    the app.
    """

    stats: Optional[Stats] = None
    """A collection of stats for the app."""

    status: AppStatuses
    """If the status is live, the app is visible on Whop discovery.

    In order to be live, you need to set the name, icon, and description. Being
    unlisted or hidden means it's not visible on Whop but you can still install the
    app via direct link. To remove the app from whop discovery, you should set the
    status to unlisted.
    """

    verified: bool
    """Whether this app has been verified by Whop.

    Verified apps are endorsed by whop and are shown in the 'featured apps' section
    of the app store.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from .currency import Currency
from ..._models import BaseModel

__all__ = ["Transfer", "Destination", "DestinationUser", "DestinationCompany", "Origin", "OriginUser", "OriginCompany"]


class DestinationUser(BaseModel):
    id: str
    """The internal ID of the user."""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    typename: Literal["User"]
    """The typename of this object"""

    username: str
    """The username of the user from their Whop account."""


class DestinationCompany(BaseModel):
    id: str
    """The ID (tag) of the company."""

    route: str
    """The slug/route of the company on the Whop site."""

    title: str
    """The title of the company."""

    typename: Literal["Company"]
    """The typename of this object"""


Destination: TypeAlias = Annotated[
    Union[Optional[DestinationUser], Optional[DestinationCompany]], PropertyInfo(discriminator="typename")
]


class OriginUser(BaseModel):
    id: str
    """The internal ID of the user."""

    name: Optional[str] = None
    """The name of the user from their Whop account."""

    typename: Literal["User"]
    """The typename of this object"""

    username: str
    """The username of the user from their Whop account."""


class OriginCompany(BaseModel):
    id: str
    """The ID (tag) of the company."""

    route: str
    """The slug/route of the company on the Whop site."""

    title: str
    """The title of the company."""

    typename: Literal["Company"]
    """The typename of this object"""


Origin: TypeAlias = Annotated[
    Union[Optional[OriginUser], Optional[OriginCompany]], PropertyInfo(discriminator="typename")
]


class Transfer(BaseModel):
    id: str
    """The unique identifier of the credit transaction transfer"""

    amount: float
    """The amount of the credit transaction transfer"""

    created_at: datetime
    """The timestamp when the credit transaction transfer was created"""

    currency: Currency
    """The currency of the credit transaction transfer"""

    destination: Destination
    """The recipient of the credit transaction transfer"""

    destination_ledger_account_id: str
    """The ID of the destination ledger account"""

    fee_amount: Optional[float] = None
    """The decimal fee of the credit transaction transfer"""

    notes: Optional[str] = None
    """The notes of the credit transaction transfer"""

    origin: Origin
    """The sender of the credit transaction transfer"""

    origin_ledger_account_id: str
    """The ID of the origin ledger account"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ShipmentCarrier"]

ShipmentCarrier: TypeAlias = Literal[
    "accurate",
    "amazon_mws",
    "amazon_shipping",
    "apc",
    "asendia_usa",
    "australia_post",
    "axlehire_v3",
    "better_trucks",
    "canada_post",
    "canpar",
    "columbus_last_mile",
    "chronopost",
    "cloud_sort",
    "courier_express",
    "couriers_please",
    "cs_logistics",
    "dai_post",
    "deutsche_post_uk",
    "deutsche_post",
    "dhl_ecommerce_asia",
    "dhl_ecs",
    "dhl_express",
    "dhl_paket",
    "door_dash",
    "dpd_nl",
    "dpd_uk",
    "dpd",
    "epost_global",
    "estafeta",
    "evri",
    "fastway",
    "fedex_cross_border",
    "fedex_default",
    "fedex_mailview",
    "fedex_smartpost",
    "fedex",
    "first_choice",
    "first_mile",
    "flexport",
    "gio",
    "gio_express",
    "gso",
    "hailify",
    "henry",
    "interlink_express",
    "jet",
    "kuroneko_yamato",
    "la_post",
    "lasership_v2",
    "loomis_express",
    "lso",
    "ontrac",
    "optima",
    "osm_worldwide",
    "parcelforce",
    "parcll",
    "passport_global",
    "post_nl",
    "purolator",
    "quick",
    "royal_mail",
    "omni_parcel",
    "sendle",
    "sf_express",
    "smart_kargo",
    "sonic",
    "spee_dee",
    "swyft",
    "tforce",
    "uds",
    "ups_iparcel",
    "ups_mail_innovations",
    "ups",
    "usps",
    "veho",
    "yanwen",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["MembershipCancelParams"]


class MembershipCancelParams(TypedDict, total=False):
    cancellation_mode: Optional[Literal["at_period_end", "immediate"]]
    """The mode of cancellation for a membership"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["BusinessTypes"]

BusinessTypes: TypeAlias = Literal[
    "education_program",
    "coaching",
    "software",
    "paid_group",
    "newsletter",
    "agency",
    "physical_products",
    "brick_and_mortar",
    "events",
    "coaching_and_courses",
    "other",
    "saas",
    "course",
    "community",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import transfer_list_params, transfer_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.currency import Currency
from ..types.shared.transfer import Transfer
from ..types.shared.direction import Direction
from ..types.transfer_list_response import TransferListResponse

__all__ = ["TransfersResource", "AsyncTransfersResource"]


class TransfersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransfersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return TransfersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransfersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return TransfersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        amount: float,
        currency: Currency,
        destination_id: str,
        origin_id: str,
        idempotence_key: Optional[str] | Omit = omit,
        notes: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Transfer:
        """
        Creates a new transfer between ledger accounts

        Required permissions:

        - `payout:transfer_funds`

        Args:
          amount: The amount to withdraw

          currency: The currency that is being withdrawn.

          destination_id: The ID of the destination account which will receive the funds (either a User
              ID, Company ID, or LedgerAccount ID)

          origin_id: The ID of the origin account which will send the funds (either a User ID,
              Company ID, or LedgerAccount ID)

          idempotence_key: A unique key to ensure idempotence. Use a UUID or similar.

          notes: Notes for the transfer. Maximum of 50 characters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/transfers",
            body=maybe_transform(
                {
                    "amount": amount,
                    "currency": currency,
                    "destination_id": destination_id,
                    "origin_id": origin_id,
                    "idempotence_key": idempotence_key,
                    "notes": notes,
                },
                transfer_create_params.TransferCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Transfer,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Transfer:
        """
        Retrieves a transfer by ID

        Required permissions:

        - `payout:transfer:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/transfers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Transfer,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        destination_id: Optional[str] | Omit = omit,
        direction: Optional[Direction] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        order: Optional[Literal["amount", "created_at"]] | Omit = omit,
        origin_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[TransferListResponse]:
        """
        Lists transfers

        Required permissions:

        - `payout:transfer:read`

        Args:
          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          destination_id: Filter transfers to only those that were sent to this destination account.
              (user_xxx, biz_xxx, ldgr_xxx)

          direction: The direction of the sort.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          order: Which columns can be used to sort.

          origin_id: Filter transfers to only those that were sent from this origin account.
              (user_xxx, biz_xxx, ldgr_xxx)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/transfers",
            page=SyncCursorPage[TransferListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "destination_id": destination_id,
                        "direction": direction,
                        "first": first,
                        "last": last,
                        "order": order,
                        "origin_id": origin_id,
                    },
                    transfer_list_params.TransferListParams,
                ),
            ),
            model=TransferListResponse,
        )


class AsyncTransfersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransfersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/whopio/whopsdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTransfersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransfersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/whopio/whopsdk-python#with_streaming_response
        """
        return AsyncTransfersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        amount: float,
        currency: Currency,
        destination_id: str,
        origin_id: str,
        idempotence_key: Optional[str] | Omit = omit,
        notes: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Transfer:
        """
        Creates a new transfer between ledger accounts

        Required permissions:

        - `payout:transfer_funds`

        Args:
          amount: The amount to withdraw

          currency: The currency that is being withdrawn.

          destination_id: The ID of the destination account which will receive the funds (either a User
              ID, Company ID, or LedgerAccount ID)

          origin_id: The ID of the origin account which will send the funds (either a User ID,
              Company ID, or LedgerAccount ID)

          idempotence_key: A unique key to ensure idempotence. Use a UUID or similar.

          notes: Notes for the transfer. Maximum of 50 characters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/transfers",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "currency": currency,
                    "destination_id": destination_id,
                    "origin_id": origin_id,
                    "idempotence_key": idempotence_key,
                    "notes": notes,
                },
                transfer_create_params.TransferCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Transfer,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Transfer:
        """
        Retrieves a transfer by ID

        Required permissions:

        - `payout:transfer:read`

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/transfers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Transfer,
        )

    def list(
        self,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        destination_id: Optional[str] | Omit = omit,
        direction: Optional[Direction] | Omit = omit,
        first: Optional[int] | Omit = omit,
        last: Optional[int] | Omit = omit,
        order: Optional[Literal["amount", "created_at"]] | Omit = omit,
        origin_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TransferListResponse, AsyncCursorPage[TransferListResponse]]:
        """
        Lists transfers

        Required permissions:

        - `payout:transfer:read`

        Args:
          after: Returns the elements in the list that come after the specified cursor.

          before: Returns the elements in the list that come before the specified cursor.

          destination_id: Filter transfers to only those that were sent to this destination account.
              (user_xxx, biz_xxx, ldgr_xxx)

          direction: The direction of the sort.

          first: Returns the first _n_ elements from the list.

          last: Returns the last _n_ elements from the list.

          order: Which columns can be used to sort.

          origin_id: Filter transfers to only those that were sent from this origin account.
              (user_xxx, biz_xxx, ldgr_xxx)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/transfers",
            page=AsyncCursorPage[TransferListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "destination_id": destination_id,
                        "direction": direction,
                        "first": first,
                        "last": last,
                        "order": order,
                        "origin_id": origin_id,
                    },
                    transfer_list_params.TransferListParams,
                ),
            ),
            model=TransferListResponse,
        )


class TransfersResourceWithRawResponse:
    def __init__(self, transfers: TransfersResource) -> None:
        self._transfers = transfers

        self.create = to_raw_response_wrapper(
            transfers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            transfers.retrieve,
        )
        self.list = to_raw_response_wrapper(
            transfers.list,
        )


class AsyncTransfersResourceWithRawResponse:
    def __init__(self, transfers: AsyncTransfersResource) -> None:
        self._transfers = transfers

        self.create = async_to_raw_response_wrapper(
            transfers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            transfers.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            transfers.list,
        )


class TransfersResourceWithStreamingResponse:
    def __init__(self, transfers: TransfersResource) -> None:
        self._transfers = transfers

        self.create = to_streamed_response_wrapper(
            transfers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            transfers.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            transfers.list,
        )


class AsyncTransfersResourceWithStreamingResponse:
    def __init__(self, transfers: AsyncTransfersResource) -> None:
        self._transfers = transfers

        self.create = async_to_streamed_response_wrapper(
            transfers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            transfers.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            transfers.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr
from .shared.who_can_post import WhoCanPost
from .shared.who_can_react import WhoCanReact

__all__ = ["ChatChannelUpdateParams"]


class ChatChannelUpdateParams(TypedDict, total=False):
    ban_media: Optional[bool]
    """Whether media uploads are banned in this chat"""

    ban_urls: Optional[bool]
    """Whether URLs are banned in this chat"""

    banned_words: Optional[SequenceNotStr[str]]
    """List of banned words for this chat"""

    user_posts_cooldown_seconds: Optional[int]
    """The cooldown period in seconds between user posts"""

    who_can_post: Optional[WhoCanPost]
    """Who can post on a chat feed"""

    who_can_react: Optional[WhoCanReact]
    """Who can react on a chat feed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from whop_sdk import Whop, AsyncWhop
from tests.utils import assert_matches_type
from whop_sdk.types import ForumPostListResponse
from whop_sdk.pagination import SyncCursorPage, AsyncCursorPage
from whop_sdk.types.shared import ForumPost

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestForumPosts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Whop) -> None:
        forum_post = client.forum_posts.create(
            experience_id="exp_xxxxxxxxxxxxxx",
        )
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Whop) -> None:
        forum_post = client.forum_posts.create(
            experience_id="exp_xxxxxxxxxxxxxx",
            attachments=[
                {
                    "id": "id",
                    "direct_upload_id": "direct_upload_id",
                }
            ],
            content="content",
            is_mention=True,
            parent_id="parent_id",
            paywall_amount=6.9,
            paywall_currency="usd",
            pinned=True,
            poll={
                "options": [
                    {
                        "id": "id",
                        "text": "text",
                    }
                ]
            },
            title="title",
        )
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Whop) -> None:
        response = client.forum_posts.with_raw_response.create(
            experience_id="exp_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forum_post = response.parse()
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Whop) -> None:
        with client.forum_posts.with_streaming_response.create(
            experience_id="exp_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forum_post = response.parse()
            assert_matches_type(ForumPost, forum_post, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Whop) -> None:
        forum_post = client.forum_posts.retrieve(
            "id",
        )
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Whop) -> None:
        response = client.forum_posts.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forum_post = response.parse()
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Whop) -> None:
        with client.forum_posts.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forum_post = response.parse()
            assert_matches_type(ForumPost, forum_post, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.forum_posts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Whop) -> None:
        forum_post = client.forum_posts.list(
            experience_id="exp_xxxxxxxxxxxxxx",
        )
        assert_matches_type(SyncCursorPage[ForumPostListResponse], forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Whop) -> None:
        forum_post = client.forum_posts.list(
            experience_id="exp_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            first=42,
            last=42,
            parent_id="parent_id",
            pinned=True,
        )
        assert_matches_type(SyncCursorPage[ForumPostListResponse], forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Whop) -> None:
        response = client.forum_posts.with_raw_response.list(
            experience_id="exp_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forum_post = response.parse()
        assert_matches_type(SyncCursorPage[ForumPostListResponse], forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Whop) -> None:
        with client.forum_posts.with_streaming_response.list(
            experience_id="exp_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forum_post = response.parse()
            assert_matches_type(SyncCursorPage[ForumPostListResponse], forum_post, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncForumPosts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncWhop) -> None:
        forum_post = await async_client.forum_posts.create(
            experience_id="exp_xxxxxxxxxxxxxx",
        )
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncWhop) -> None:
        forum_post = await async_client.forum_posts.create(
            experience_id="exp_xxxxxxxxxxxxxx",
            attachments=[
                {
                    "id": "id",
                    "direct_upload_id": "direct_upload_id",
                }
            ],
            content="content",
            is_mention=True,
            parent_id="parent_id",
            paywall_amount=6.9,
            paywall_currency="usd",
            pinned=True,
            poll={
                "options": [
                    {
                        "id": "id",
                        "text": "text",
                    }
                ]
            },
            title="title",
        )
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncWhop) -> None:
        response = await async_client.forum_posts.with_raw_response.create(
            experience_id="exp_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forum_post = await response.parse()
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncWhop) -> None:
        async with async_client.forum_posts.with_streaming_response.create(
            experience_id="exp_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forum_post = await response.parse()
            assert_matches_type(ForumPost, forum_post, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncWhop) -> None:
        forum_post = await async_client.forum_posts.retrieve(
            "id",
        )
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncWhop) -> None:
        response = await async_client.forum_posts.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forum_post = await response.parse()
        assert_matches_type(ForumPost, forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncWhop) -> None:
        async with async_client.forum_posts.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forum_post = await response.parse()
            assert_matches_type(ForumPost, forum_post, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.forum_posts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncWhop) -> None:
        forum_post = await async_client.forum_posts.list(
            experience_id="exp_xxxxxxxxxxxxxx",
        )
        assert_matches_type(AsyncCursorPage[ForumPostListResponse], forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncWhop) -> None:
        forum_post = await async_client.forum_posts.list(
            experience_id="exp_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            first=42,
            last=42,
            parent_id="parent_id",
            pinned=True,
        )
        assert_matches_type(AsyncCursorPage[ForumPostListResponse], forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncWhop) -> None:
        response = await async_client.forum_posts.with_raw_response.list(
            experience_id="exp_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forum_post = await response.parse()
        assert_matches_type(AsyncCursorPage[ForumPostListResponse], forum_post, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncWhop) -> None:
        async with async_client.forum_posts.with_streaming_response.list(
            experience_id="exp_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forum_post = await response.parse()
            assert_matches_type(AsyncCursorPage[ForumPostListResponse], forum_post, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["FriendlyReceiptStatus"]

FriendlyReceiptStatus: TypeAlias = Literal[
    "auto_refunded",
    "refunded",
    "partially_refunded",
    "dispute_warning",
    "open_resolution",
    "open_dispute",
    "failed",
    "price_too_low",
    "succeeded",
    "drafted",
    "uncollectible",
    "unresolved",
    "past_due",
    "pending",
    "incomplete",
    "canceled",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["AccessLevel"]

AccessLevel: TypeAlias = Literal["no_access", "admin", "customer"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from whop_sdk import Whop, AsyncWhop
from tests.utils import assert_matches_type
from whop_sdk.types import (
    PlanListResponse,
    PlanDeleteResponse,
)
from whop_sdk.pagination import SyncCursorPage, AsyncCursorPage
from whop_sdk.types.shared import Plan

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPlans:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Whop) -> None:
        plan = client.plans.create(
            company_id="biz_xxxxxxxxxxxxxx",
            product_id="prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Whop) -> None:
        plan = client.plans.create(
            company_id="biz_xxxxxxxxxxxxxx",
            product_id="prod_xxxxxxxxxxxxx",
            billing_period=42,
            currency="usd",
            custom_fields=[
                {
                    "field_type": "text",
                    "name": "name",
                    "id": "id",
                    "order": 42,
                    "placeholder": "placeholder",
                    "required": True,
                }
            ],
            description="description",
            expiration_days=42,
            image={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            initial_price=6.9,
            internal_notes="internal_notes",
            override_tax_type="inclusive",
            plan_type="renewal",
            release_method="buy_now",
            renewal_price=6.9,
            title="title",
            trial_period_days=42,
            visibility="visible",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Whop) -> None:
        response = client.plans.with_raw_response.create(
            company_id="biz_xxxxxxxxxxxxxx",
            product_id="prod_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Whop) -> None:
        with client.plans.with_streaming_response.create(
            company_id="biz_xxxxxxxxxxxxxx",
            product_id="prod_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(Plan, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Whop) -> None:
        plan = client.plans.retrieve(
            "plan_xxxxxxxxxxxxx",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Whop) -> None:
        response = client.plans.with_raw_response.retrieve(
            "plan_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Whop) -> None:
        with client.plans.with_streaming_response.retrieve(
            "plan_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(Plan, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.plans.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Whop) -> None:
        plan = client.plans.update(
            id="plan_xxxxxxxxxxxxx",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Whop) -> None:
        plan = client.plans.update(
            id="plan_xxxxxxxxxxxxx",
            billing_period=42,
            currency="usd",
            custom_fields=[
                {
                    "field_type": "text",
                    "name": "name",
                    "id": "id",
                    "order": 42,
                    "placeholder": "placeholder",
                    "required": True,
                }
            ],
            description="description",
            expiration_days=42,
            image={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            initial_price=6.9,
            internal_notes="internal_notes",
            offer_cancel_discount=True,
            override_tax_type="inclusive",
            renewal_price=6.9,
            title="title",
            trial_period_days=42,
            visibility="visible",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Whop) -> None:
        response = client.plans.with_raw_response.update(
            id="plan_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Whop) -> None:
        with client.plans.with_streaming_response.update(
            id="plan_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(Plan, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.plans.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Whop) -> None:
        plan = client.plans.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Whop) -> None:
        plan = client.plans.list(
            company_id="biz_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            direction="asc",
            first=42,
            last=42,
            order="id",
            plan_types=["renewal"],
            product_ids=["string"],
            release_methods=["buy_now"],
            visibilities=["visible"],
        )
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Whop) -> None:
        response = client.plans.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Whop) -> None:
        with client.plans.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(SyncCursorPage[PlanListResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Whop) -> None:
        plan = client.plans.delete(
            "plan_xxxxxxxxxxxxx",
        )
        assert_matches_type(PlanDeleteResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Whop) -> None:
        response = client.plans.with_raw_response.delete(
            "plan_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanDeleteResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Whop) -> None:
        with client.plans.with_streaming_response.delete(
            "plan_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanDeleteResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Whop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.plans.with_raw_response.delete(
                "",
            )


class TestAsyncPlans:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncWhop) -> None:
        plan = await async_client.plans.create(
            company_id="biz_xxxxxxxxxxxxxx",
            product_id="prod_xxxxxxxxxxxxx",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncWhop) -> None:
        plan = await async_client.plans.create(
            company_id="biz_xxxxxxxxxxxxxx",
            product_id="prod_xxxxxxxxxxxxx",
            billing_period=42,
            currency="usd",
            custom_fields=[
                {
                    "field_type": "text",
                    "name": "name",
                    "id": "id",
                    "order": 42,
                    "placeholder": "placeholder",
                    "required": True,
                }
            ],
            description="description",
            expiration_days=42,
            image={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            initial_price=6.9,
            internal_notes="internal_notes",
            override_tax_type="inclusive",
            plan_type="renewal",
            release_method="buy_now",
            renewal_price=6.9,
            title="title",
            trial_period_days=42,
            visibility="visible",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncWhop) -> None:
        response = await async_client.plans.with_raw_response.create(
            company_id="biz_xxxxxxxxxxxxxx",
            product_id="prod_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncWhop) -> None:
        async with async_client.plans.with_streaming_response.create(
            company_id="biz_xxxxxxxxxxxxxx",
            product_id="prod_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(Plan, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncWhop) -> None:
        plan = await async_client.plans.retrieve(
            "plan_xxxxxxxxxxxxx",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncWhop) -> None:
        response = await async_client.plans.with_raw_response.retrieve(
            "plan_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncWhop) -> None:
        async with async_client.plans.with_streaming_response.retrieve(
            "plan_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(Plan, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.plans.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncWhop) -> None:
        plan = await async_client.plans.update(
            id="plan_xxxxxxxxxxxxx",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncWhop) -> None:
        plan = await async_client.plans.update(
            id="plan_xxxxxxxxxxxxx",
            billing_period=42,
            currency="usd",
            custom_fields=[
                {
                    "field_type": "text",
                    "name": "name",
                    "id": "id",
                    "order": 42,
                    "placeholder": "placeholder",
                    "required": True,
                }
            ],
            description="description",
            expiration_days=42,
            image={
                "id": "id",
                "direct_upload_id": "direct_upload_id",
            },
            initial_price=6.9,
            internal_notes="internal_notes",
            offer_cancel_discount=True,
            override_tax_type="inclusive",
            renewal_price=6.9,
            title="title",
            trial_period_days=42,
            visibility="visible",
        )
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncWhop) -> None:
        response = await async_client.plans.with_raw_response.update(
            id="plan_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(Plan, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncWhop) -> None:
        async with async_client.plans.with_streaming_response.update(
            id="plan_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(Plan, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.plans.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncWhop) -> None:
        plan = await async_client.plans.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncWhop) -> None:
        plan = await async_client.plans.list(
            company_id="biz_xxxxxxxxxxxxxx",
            after="after",
            before="before",
            direction="asc",
            first=42,
            last=42,
            order="id",
            plan_types=["renewal"],
            product_ids=["string"],
            release_methods=["buy_now"],
            visibilities=["visible"],
        )
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncWhop) -> None:
        response = await async_client.plans.with_raw_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncWhop) -> None:
        async with async_client.plans.with_streaming_response.list(
            company_id="biz_xxxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(AsyncCursorPage[PlanListResponse], plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncWhop) -> None:
        plan = await async_client.plans.delete(
            "plan_xxxxxxxxxxxxx",
        )
        assert_matches_type(PlanDeleteResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncWhop) -> None:
        response = await async_client.plans.with_raw_response.delete(
            "plan_xxxxxxxxxxxxx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanDeleteResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncWhop) -> None:
        async with async_client.plans.with_streaming_response.delete(
            "plan_xxxxxxxxxxxxx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanDeleteResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncWhop) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.plans.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ShipmentCreateParams"]


class ShipmentCreateParams(TypedDict, total=False):
    company_id: Required[str]
    """The ID of the company to create the shipment for"""

    payment_id: Required[str]
    """The ID of the payment to create the shipment for"""

    tracking_code: Required[str]
    """The tracking code for the shipment"""
